<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Request a Quote GraphQl for Magento 2 (System)
 */

namespace Amasty\RequestAQuoteGraphql\Model\Resolver;

use Amasty\RequestAQuoteGraphql\Model\QuoteCart\GetQuoteForUser;
use Amasty\RequestAQuoteGraphql\Model\QuoteCart\IsEnabledForCustomer as IsQuoteCartEnabledProvider;
use Amasty\RequestQuote\Api\QuoteRepositoryInterface;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\GraphQl\Model\Query\ContextInterface;

class ClearQuote implements ResolverInterface
{
    /**
     * @var IsQuoteCartEnabledProvider
     */
    private $isQuoteCartEnabled;

    /**
     * @var GetQuoteForUser
     */
    private $getQuoteForUser;

    /**
     * @var QuoteRepositoryInterface
     */
    private $quoteRepository;

    public function __construct(
        IsQuoteCartEnabledProvider $isQuoteCartEnabled,
        GetQuoteForUser $getQuoteForUser,
        QuoteRepositoryInterface $quoteRepository
    ) {
        $this->isQuoteCartEnabled = $isQuoteCartEnabled;
        $this->getQuoteForUser = $getQuoteForUser;
        $this->quoteRepository = $quoteRepository;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws GraphQlAuthorizationException
     * @throws GraphQlInputException
     *
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function resolve(Field $field, $context, ResolveInfo $info, array $value = null, array $args = null)
    {
        if (!$this->isQuoteCartEnabled->execute($context->getExtensionAttributes()->getCustomerGroupId())) {
            throw new GraphQlAuthorizationException(__('Quote Cart has been disabled.'));
        }
        if (empty($args['quoteId'])) {
            throw new GraphQlInputException(__('Required parameter "cartId" is missing'));
        }

        $quote = $this->getQuoteForUser->execute(
            $args['quoteId'],
            $context->getUserId(),
            (int) $context->getExtensionAttributes()->getStore()->getId()
        );
        $quote->removeAllItems();
        $quote->collectTotals();
        $this->quoteRepository->save($quote);

        return [
            'model' => $quote
        ];
    }
}
